import pyvisa
import numpy as np
import time
import xdrlib
import logging
import socket
import vxi11

consoleHandler = logging.StreamHandler()
logger = logging.getLogger("spd3303x")
logger.addHandler(consoleHandler)
logger.setLevel(logging.DEBUG)

class SPD3303X(object):

    KNOWN_MODELS = [
        "SPD3303X",
        "SPD3303X-E",
        "SPD3XIDD5R7170",
    ]

    MANUFACTURERS = {
        "SPD3303X": "Siglent",
        "SPD3303X-E": "Siglent",
        # This one is called RSPD3303X-E by RS PRO.
        "SPD3XIDD5R7170": "[RS PRO]",
    }


    @classmethod
    def usb_device(cls, visa_rscr: str=None):
        return USBDevice(visa_rscr)

    @classmethod
    def ethernet_device(cls, host: str):
        return EthernetDevice(host)

    class Channel(object):
        def __init__(self, chan_no: int, dev):
            self._name = f"CH{chan_no}"
            self._dev = dev

        def set_output(self, status: bool):
            self._dev.write(f"OUTP {self._name},{'ON' if status else 'OFF'}")

    class ControlledChannel(Channel):
        def set_voltage(self, voltage: float):
            self._dev.write(f"{self._name}:VOLT {voltage:.3f}")

        def set_current(self, current: float):
            self._dev.write(f"{self._name}:CURR {current:.3f}")

        def get_voltage(self):
            return self._dev.query(f"{self._name}:VOLT?")

        def get_current(self):
            return self._dev.query(f"{self._name}:CURR?")

        def measure_voltage(self):
            return self._dev.query(f"MEAS:VOLT? {self._name}")

        def measure_current(self):
            return self._dev.query(f"MEAS:CURR? {self._name}")

        def measure_power(self):
            return self._dev.query(f"MEAS:POWE? {self._name}")

    def __enter__(self):
        try:
            dsc = self.query("*IDN?")
        except pyvisa.errors.VisaIOError:
            self._inst.close()
            raise
        identity_items = dsc.split(",")
        if len(identity_items) == 3:
            # RS PRO RSPD3303X-E ?
            model, _, _= dsc.split(",")
            mnf = self.MANUFACTURERS.get(model, "[Unknown]")
        else:
            # Proper Siglent device probably.
            mnf, model,_,_,_ = identity_items
        logger.debug(f"Discovered {model} by {mnf}")
        if model not in self.KNOWN_MODELS:
            raise Exception(f"Device {model} not supported")
        self.CH1 = SPD3303X.ControlledChannel(1, self)
        self.CH2 = SPD3303X.ControlledChannel(2, self)
        self.CH3 = SPD3303X.Channel(3, self)
        return self

    def __exit__(self, *args):
        self._inst.close()

class USBDevice(SPD3303X):
    def __init__(self, visa_rscr: str=None):
        self._visa_rscr = visa_rscr

    def __enter__(self):
        rm = pyvisa.ResourceManager("@py")
        if self._visa_rscr is None:
            logger.debug("Trying to auto-detect USB device")
            resources = rm.list_resources()
            for res_str in resources:
                if "SPD3XID" in res_str:
                    self._visa_rscr = res_str
            if self._visa_rscr is None:
                raise Exception("No device found")

        self._inst = rm.open_resource(self._visa_rscr)
        self._inst.write_termination="\n"
        self._inst.read_termination="\n"
        return super().__enter__()

    def write(self, cmd: str):
        self._inst.write(cmd)
        time.sleep(0.1)

    def query(self, cmd: str):
        self.write(cmd)
        rep = self._inst.read()
        time.sleep(0.1)
        return rep

class EthernetDevice(SPD3303X):
    def __init__(self, host: str):
        self._host = host

    def __enter__(self):
        try:
            logger.debug(f"Trying to resolve host {self._host}")
            ip_addr = socket.gethostbyname(self._host)
        except socket.gaierror:
            logger.error(f"Couldn't resolve host {self._host}")

        self._inst = vxi11.Instrument(ip_addr)
        return super().__enter__()

    def write(self, cmd: str):
        self._inst.write(cmd)
        time.sleep(0.1)

    def query(self, cmd: str):
        return self._inst.ask(cmd)





import click
import logging
import re
from spd3303x import SPD3303X

logger = logging.getLogger("spd3303x")

@click.group(context_settings=dict(help_option_names=["-h", "--help"], obj={}))
@click.option("-v", "--verbose", count=True, default=2)
@click.option("-d", "--device", type=str)
@click.pass_context
def cli(ctx, verbose, device):


    if verbose == 0:
        logger.setLevel(logging.ERROR)
    elif verbose == 1:
        logger.setLevel(logging.WARNING)
    elif verbose == 2:
        logger.setLevel(logging.INFO)
    elif verbose > 2:
        logger.setLevel(logging.DEBUG)

    ctx.obj["verbose"] = verbose

    if device is None:
        logger.info("No device specified. Trying USB.")
        ctx.obj["device"] = SPD3303X.usb_device()
    else:
        if re.match("USB\d?::.*", device) is None:
            logger.debug(f"Assuming Ethernet device at {device}")
            ctx.obj["device"] = SPD3303X.ethernet_device(device)
        else:
            logger.debug(f"Assuming USB device at {device}")
            ctx.obj["device"] = SPD3303X.usb_device(device)


@cli.command(short_help="Set channel")
@click.argument("channel", type=int)
@click.option("--voltage", "-v", type=float, help="Voltage")
@click.option("--current", "-c", type=float, help="Current")
@click.option("--on/--off", default=False, help="Current")
@click.pass_context
def set(ctx, channel, voltage, current, on):
    with ctx.obj["device"] as dev:
        channel_name = f"CH{channel}"
        channel = getattr(dev, channel_name)
        channel.set_output(False)
        if voltage is not None:
            try:
                channel.set_voltage(voltage)
            except AttributeError:
                raise click.UsageError(f"{channel_name} does not support setting voltage")

        if current is not None:
            try:
                channel.set_current(current)
            except AttributeError:
                raise click.UsageError(f"{channel_name} does not support setting current")
        channel.set_output(on)
